/**
 * Created by aaronlin on 2017/7/18.
 */
import React, { PureComponent } from 'react';

import PropTypes from 'prop-types';

import {
    Animated,
    TouchableWithoutFeedback,
    Platform,
    Dimensions,
    ViewPropTypes,
} from 'react-native';


import WMText from './WMText';
import WMView from './WMView';
import WMImage from './WMImage';
import HtmlView from './html/HtmlView';

import {
    ImageSourcePropType
} from './RNExt';

const CHILD_VIEW_HEIGHT = 50;

const { fontScale } = Dimensions.get('window');

/**
 *
 */
export default class NoticeViewV2 extends PureComponent{

    static propTypes = {
        ...ViewPropTypes,
        fontSize:PropTypes.number,  //文字大小
        fontColor:PropTypes.string,    //文字颜色
        headIcon:ImageSourcePropType,  //文字头部的 image source
        footerIcon:ImageSourcePropType,  //文字尾部的 image source
        dataList:PropTypes.arrayOf(PropTypes.shape({    //数据
            title:PropTypes.string.isRequired, //用来显示的name
            targetObj:PropTypes.object    //每个item对应的自定义数据
        })),
    }

    static defaultProps = {
        fontSize:12,
        fontColor:'#333333',
        dataList:[],
    }

    /**
     * 是否正在执行动画
     * @type {boolean}
     */
    _isAniming = false;
    /**
     * 数据索引
     * @type {number}
     * @private
     */
    _dataIndex = 0;
    /**
     * 高度
     * @type {number}
     * @private
     */
    _height = CHILD_VIEW_HEIGHT;
    /**
     *
     * @type {null}
     * @private
     */
    _startAnimTimeout = null;
    /**
     * 动画时间
     * @type {number}
     * @private
     */
    _duration = 3000;
    /**
     *
     * @type {null}
     * @private
     */
    _animationFrameId = null;

    constructor(props){
        super(props);
        this.state = {
            translateYOne:new Animated.Value(0),
            translateYTwo:new Animated.Value(0),
        };
        
        const { style } = this.props;
        if(style){
            this._height = style.height || CHILD_VIEW_HEIGHT;
        }
    }

    /**
     * 开始动画
     * @private
     */
    _startAnim = ()=>{
        const { dataList = [] } = this.props;

        if(dataList.length <= 1){
            this._dataIndex = 0;
            return ;
        }
        if(this._dataIndex >= dataList.length){
            this._dataIndex = 0;
        }
        let data = dataList[this._dataIndex];
        let htmlTxt = null;
        if(data){
            htmlTxt = '<font>' + data.title +'</font>';
        }
        if(this.state.translateYOne._value == this._height){
            if(htmlTxt){
                this.refs.htmlViewOne && this.refs.htmlViewOne.startHtmlRender(htmlTxt);
                this.refs.viewChildOne && this.refs.viewChildOne.setTag({data:data, index:this._dataIndex});
            }
        }else if(this.state.translateYTwo._value == 0){
            if(htmlTxt){
                this.refs.htmlViewTwo && this.refs.htmlViewTwo.startHtmlRender(htmlTxt);
                this.refs.viewChildTwo && this.refs.viewChildTwo.setTag({data:data, index:this._dataIndex});
            }
        }
        this._isAniming = true;
        Animated.parallel([
            Animated.timing(this.state.translateYOne, {
                fromValue: this.state.translateYOne._value,
                toValue: this.state.translateYOne._value - this._height,
                duration: this._duration,
                isInteraction: false,
            }),
            Animated.timing(this.state.translateYTwo, {
                fromValue: this.state.translateYTwo._value,
                toValue: this.state.translateYTwo._value - this._height,
                duration: this._duration,
                isInteraction: false,
            }),
        ]).start(()=>{
            if(this.state.translateYOne._value <= -this._height){
                this.state.translateYOne.setValue(this._height);
            }
            if(this.state.translateYTwo._value <= -this._height * 2){
                this.state.translateYTwo.setValue(0);
            }
            if(this._isAniming){
                this._dataIndex++;
                this._animationFrameId = requestAnimationFrame(this._startAnim);
            }
        });
    }

    /**
     *
     */
    startAnim = ()=>{
        this._startAnimTimeout = setTimeout(()=>{
            if(!this._isAniming){
                this._startAnim();
            }
        });
    }

    /**
     * 停止动画
     */
    stopAnim = () => {

        this._isAniming = false;

        this._animationFrameId && cancelAnimationFrame(this._animationFrameId);
        this._animationFrameId = null;

        this.state.translateYOne.stopAnimation();
        this.state.translateYTwo.stopAnimation();
        //恢复位置
        this.state.translateYOne.setValue(0);
        this.state.translateYTwo.setValue(0);
    }

    componentDidMount() {
        this._startAnim();
    }

    componentDidUpdate() {
    }

    componentWillUnmount() {
        this._startAnimTimeout && clearTimeout(this._startAnimTimeout);
        this.stopAnim();
    }

    /**
     *
     * @param e
     * @private
     */
    _onItemClick = (e)=>{
        let ref = null;
        try {
            ref = e._targetInst.memoizedProps.refName;
        } catch (error){

        }

        let tag = undefined;
        if(ref){
            tag = this.refs[ref].getTag();
        }
        this.props.onChange && this.props.onChange(tag);
    }

    render(){
        const WMMView = Animated.createAnimatedComponent(WMView);

        let htmlTxt = null;
        const { dataList = [], fontSize, fontColor, headIcon, footerIcon, style } = this.props;
        let data = dataList[this._dataIndex];
        let index = this._dataIndex;
        if(data){
            htmlTxt = '<font>' + data.title +'</font>';
            if (dataList.length > 1) {
                this._dataIndex++;
            }
        }

        return (
            <WMMView ref='rootView' style={[styles.rootContainer, {height:this._height, borderRadius: this._height / 2}, style]}>
                <WMMView style={[{transform:[{translateY:this.state.translateYOne}]}]}>
                    <TouchableWithoutFeedback onPress={this._onItemClick}>
                        <WMView ref='viewChildOne' refName={'viewChildOne'} tag={{data:data, index:index}} style={[styles.itemContainer, {height:this._height}]}>
                            <WMImage ref='imageOne' style={styles.image} source={headIcon}/>
                            <HtmlView ref='htmlViewOne' stylesheet={{font:{fontSize: fontSize, color: fontColor}}} style={[styles.title, {height:this._height}]} value={htmlTxt}/>
                            <WMImage ref='accessImageOne' style={styles.accessImage} source={footerIcon} />
                        </WMView>
                    </TouchableWithoutFeedback>
                </WMMView>

                <WMMView style={[{transform:[{translateY:this.state.translateYTwo}]}]}>
                    <TouchableWithoutFeedback onPress={this._onItemClick}>
                        <WMView ref='viewChildTwo' refName={'viewChildTwo'} tag={{data:data, index:index}} style={[styles.itemContainer, {height:this._height}]}>
                            <WMImage ref='imageTwo' style={styles.image} source={headIcon}/>
                            <HtmlView ref='htmlViewTwo' stylesheet={{font:{fontSize: fontSize, color: fontColor}}} style={[styles.title, {height:this._height}]} value={htmlTxt}/>
                            <WMImage ref='accessImageTwo' style={styles.accessImage} source={footerIcon} />
                        </WMView>
                    </TouchableWithoutFeedback>
                </WMMView>
            </WMMView>
        );
    }

}

const styles = {
    rootContainer:{
        backgroundColor:'white',
        overflow:'hidden',
        marginLeft: 15, 
		marginRight: 15,
    },
    itemContainer: {
        flexDirection: 'row',
        alignItems:'center'
    },
    title: {
        marginLeft: 5,
        marginRight: 10,
        justifyContent:'center',
        flex: 1,
    },
    image: {
        marginLeft: 10,
        width: 20,
        height: 20,
        resizeMode: 'contain'
    },
    accessImage: {
        marginRight: 10,
        marginLeft: 10,
        width: 6,
        height: 11,
        resizeMode: 'contain'
    }
};
