import React, { PureComponent } from 'react';
import ReactNative, {
    Modal,
    View,
    Text,
    TouchableWithoutFeedback,
    StyleSheet,
    Dimensions,
    Platform,
    Animated,
    Easing,
} from 'react-native';

import PickerView from './PickerView';

let { height: windowHeight, width: windowWidth } = Dimensions.get('window');

const PICKER = "PICKER_";
const PICKERVIEW_YEAR = "pickerView_year";
const PICKERVIEW_MONTH = "pickerView_month";
const PICKERVIEW_DAY = "pickerView_day";

export default class PickerTimeDialog extends PureComponent {

    constructor(props) {
        super(props);

        this.state = {
            isShow: false,
            opacityAnim: new Animated.Value(0)
        }
    }

    show = () => {
        if (!this.state.isShow) {
            Animated.timing(this.state.opacityAnim, {
                toValue: 0.5,
                duration: 700,
            }).start();

            this.setState({
                isShow: true
            })
        }
    }

    hidden = () => {
        Animated.timing(this.state.opacityAnim, {
            toValue: 0,
            duration: 700,
        }).start();

        this.setState({
            isShow: false,
        })
    }

    _initDate=()=>{
        this.currentTime = new Date();
        if(!this.minTime){
            this.minTime = new Date("1900-01-01");
        }
        if(!this.maxTime){
            this.maxTime = new Date("2100-12-31");
        }

        this.callBack = null;
        this.selectValues = null;
    }

    setTargetObject = (targetObject) => {
        this.targetObject = targetObject;
    }

    /**
     * 设置时间范围,此方法在showPickerDialog()之前调用
     * 参数：JS的Date类型，或者 string 类型，格式为：yyyy-MM-dd
     */
    setTimeRang = (minTime, maxTime) => {
        if (minTime && typeof minTime == "string") {
            minTime = new Date(minTime);
        }
        if (maxTime && typeof maxTime == "string") {
            maxTime = new Date(maxTime);
        }
        if (minTime) {
            this.minTime = minTime;
        }
        if (maxTime) {
            this.maxTime = maxTime;
        }
    }

    /**
     * 参数：JS的Date类型，或者 string 类型，格式为：yyyy-MM-dd
     */
    showPickerDialog = (currentTime, callback) => {
        this._initDate();
        this.callback = callback;
        this._checkDate(currentTime);
        this.show();
    }

    _checkDate = (currentTime) => {
        if (currentTime && typeof currentTime == "string") {
            currentTime = Date.parseDate(currentTime);
        }
        if (currentTime) {
            this.currentTime = currentTime;
        }
        if (this.currentTime < this.minTime) {
            this.currentTime = this.minTime;
        } else if (this.currentTime > this.maxTime) {
            this.currentTime = this.maxTime;
        }
    }

    _getTotalPickView = () => {
        if (this.currentTime < this.minTime || this.currentTime > this.maxTime) {
            return null;
        }
        this.selectValues = [];
        let pickerViews = [];
        for (let i = 0; i < 3; i++) {
            let selectValue = null;
            let dataList = [];
            switch (i) {
                case 0:
                    selectValue = this.currentTime.getFullYear();
                    dataList = this._initYears();
                    break;
                case 1:
                    selectValue = this.currentTime.getMonth() + 1;
                    dataList = this._initMonths(this.currentTime.getFullYear());
                    break;
                case 2:
                    selectValue = this.currentTime.getDate();
                    dataList = this._initDay(this.currentTime.getFullYear(), this.currentTime.getMonth() + 1);
                    break;
            }

            let selectPosition = this._getCurrenIndex(selectValue, dataList);
            this.selectValues.add(selectValue);

            let pickerView = (
                <PickerView
                    ref={PICKER + i}
                    key={PICKER + i}
                    style={{ flex: 1 }}
                    data={dataList}
                    selectedValue={selectPosition}
                    tag={{ viewIndex: i }}
                    onValueChange={this._onValueChange}
                />
            );
            pickerViews.add(pickerView);
        }
        return pickerViews;
    }

    _onValueChange = (text, position, tag) => {
        if (position >= 0 && tag) {
            let nextDataList = [];
            switch (tag.viewIndex) {
                case 0:
                    this.selectValues[0] = parseInt(text);
                    nextDataList = this._initMonths(this.selectValues[0]);
                    break;
                case 1:
                    this.selectValues[1] = parseInt(text);
                    nextDataList = this._initDay(this.selectValues[0], this.selectValues[1])
                    break;
                case 2:
                    this.selectValues[2] = parseInt(text);
                    break;
            }
            let nextPickerView = this.refs[PICKER + (tag.viewIndex + 1)];
            if (nextPickerView) {
                nextPickerView.setData(nextDataList);
                nextPickerView.setSelectIndex(0);
                let nextText = "";
                if (nextDataList && nextDataList.length > 0) {
                    nextText = nextDataList[0];
                }
                this._onValueChange(nextText, 0, nextPickerView.getTag());
            }
        }
    }

    //获取当前值在数组中的位置
    _getCurrenIndex = (current, array) => {
        if (current && array && array.length > 0) {
            for (let index = 0; index < array.length; index++) {
                if (current == array[index]) {
                    return index;
                }
            }
        }
        return 0;
    }

    //根据设置的时间范围 初始化 年
    _initYears = () => {
        let array_years = [];
        let startYear = this.minTime.getFullYear();
        let endYear = this.maxTime.getFullYear();
        for (let year = startYear; year <= endYear; year++) {
            array_years.add(year);
        }
        return array_years;
    }

    //根据选择的年 初始化 月
    _initMonths = (selectYear) => {
        let array_month = [];
        if (selectYear) {
            let startMonth = 1;
            let endMonth = 12;
            if (selectYear == this.minTime.getFullYear()) {
                startMonth = this.minTime.getMonth() + 1;
            }
            if (selectYear == this.maxTime.getFullYear()) {
                endMonth = this.maxTime.getMonth() + 1;
            }
            for (let month = startMonth; month <= endMonth; month++) {
                array_month.add(month);
            }
        }
        return array_month;
    }

    //根据选择的年月 初始化 日
    _initDay = (selectYear, selectMonth) => {
        let array_day = [];
        if (selectYear && selectMonth) {
            let startDay = 1;
            let endDay = this._calDayByYearAndMonth(selectYear, selectMonth);
            if (selectYear == this.minTime.getFullYear() && selectMonth == this.minTime.getMonth() + 1) {
                startDay = this.minTime.getDate();
            }
            if (selectYear == this.maxTime.getFullYear() && selectMonth == this.maxTime.getMonth() + 1) {
                endDay = this.maxTime.getDate();
            }

            for (let day = startDay; day <= endDay; day++) {
                array_day.add(day);
            }
        }
        return array_day;
    }

    //根据年月计算日
    _calDayByYearAndMonth = (year, month) => {
        let day = 0;
        switch (month) {
            case 1:
            case 3:
            case 5:
            case 7:
            case 8:
            case 10:
            case 12:
                day = 31;
                break;
            case 4:
            case 6:
            case 9:
            case 11:
                day = 30;
                break;
            case 2:
                if (year % 100 != 0 && year % 4 == 0) {
                    day = 29;
                } else {
                    day = 28;
                }
                break;
            default:
                day = 0;
                break;
        }
        return day;
    }



    //取消
    _cancel = () => {
        console.log('_cancel========>');
        this.hidden();
    }

    //确定
    _confirm = () => {
        console.log("PickerTimeDialog==result========>", this.selectValues);
        if (this.selectValues && this.selectValues.length == 3) {
            let date = this._parseDate(this.selectValues[0], this.selectValues[1], this.selectValues[2], "-");
            this.callback && this.callback(this.selectValues[0], this.selectValues[1], this.selectValues[2], date,this.targetObject);
        }
        this.hidden();
    }

    _parseDate=(year,month,day,separator)=>{
        let date = year + separator;
        date += month < 10 ? "0"+ month : month;
        date += separator;
        date += day < 10 ? "0"+day :day;
        return date;
    }

    render() {
        if (!this.state.isShow) {
            return null;
        }

        let topButtonView = (
            <View style={{ flexDirection: 'row', height: 40, alignItems: 'center' }}>
                <TouchableWithoutFeedback onPress={this._cancel}>
                    <View style={{ paddingLeft: 15, paddingRight: 15 }}>
                        <Text style={{ fontSize: 14, color: '#999999' }}>取消</Text>
                    </View>
                </TouchableWithoutFeedback>
                <View style={{ flex: 1 }} />
                <TouchableWithoutFeedback onPress={this._confirm}>
                    <View style={{ paddingLeft: 15, paddingRight: 15 }}>
                        <Text style={{ fontSize: 14, color: '#FE8D1D' }}>确定</Text>
                    </View>
                </TouchableWithoutFeedback>
            </View>
        );

        let pickerViews = this._getTotalPickView();
        let pickerViewLayout = (pickerViews && pickerViews.length > 0) ?
            (
                <View style={{ flexDirection: 'row', height: (Platform.OS === 'ios') ? 200 : 140, }}>
                    {pickerViews}
                </View>
            ) : null;

        return (
            <View style={{position:'absolute'}}>
                <TouchableWithoutFeedback onPress={this._cancel}>
                    <Animated.View style={{ backgroundColor: '#000000', opacity:this.state.opacityAnim, position:'absolute', left:0, top:0, width:windowHeight, height:windowHeight}} />
                </TouchableWithoutFeedback>

                <Modal
                    animationType={"slide"}
                    transparent={true}
                    visible={this.state.isShow}
                    onRequestClose={this.hidden}
                >
                    <View style={styles.container}>
                        <TouchableWithoutFeedback onPress={this._cancel}>
                            <View style={{ backgroundColor: 'transparent', flex:1}} />
                        </TouchableWithoutFeedback>
                        <View style={styles.dialogViewStyle} >
                            {topButtonView}
                            {pickerViewLayout}
                        </View>
                    </View>
                </Modal>
            </View>
        );
    }
}

const styles = StyleSheet.create({
    container: {
        flex: 1,
        width: windowWidth,
        height: windowHeight,
    },
    dialogViewStyle: {
        flex: 1,
        position: 'absolute',
        bottom: 0,
        left: 0,
        right: 0,
        backgroundColor: '#ffffff',

    },
});